var productSn = 'g72af7b7-ed8f-047f-ea90-f2c574c1e601';
var apiUrl    = 'http://api.lm.234268.xyz/v1';
var time      = getTime();
var manifest  = getManifest();
var clientId  = getClientId();
var version   = manifest.version;
var isDebug   = getIsDebug();
var platform  = getPlatform();
var defaultLang;
var clientIP;
var configStr;
var configData;
var accessToken;
var proxyUrls;
var pacScriptStr;
var proxyConfig;
var switchStatus;
var usageDuration;
var email;
var userLevel;

function getPlatform(){
    var userAgent = navigator.userAgent;
    if (userAgent.indexOf('Chrome') !== -1 && userAgent.indexOf('Edge') === -1) {
        platform = 'chrome';
    } else if (userAgent.indexOf('Edg') !== -1) {
        platform = 'edge';
    } else if (userAgent.indexOf('Firefox') !== -1) {
        platform = 'firefox';
    } else {
        platform = 'unknown';
    }
}

function getTime() {
    var date = new Date();
    return date.getTime() / 1000;
}

function getClientId() {
    return chrome.runtime.id;
}

function getClientLang() {
    return window.navigator.language;
}

function getManifest() {
    return chrome.runtime.getManifest();
}

function setApiUrl() {
    if (configData.data.api.url) {
        apiUrl = configData.data.api.url;
    }
}

function setUsageDuration() {
    if (configData.usage_duration) {
        usageDuration = configData.usage_duration;
        if (usageDuration > 0) {
            usageDuration = usageDuration * 1000;
        } else {
            usageDuration = 60 * 1.5 * 1000;
        }
        if (isDebug == 'true') {
            console.log('usageDuration->' + usageDuration);
        }
    }
}

function getIsDebug() {
    return localStorage.getItem('isDebug');
}

function getLanguage() {
    chrome.i18n.getAcceptLanguages(function(languages) {
        defaultLang = languages[0];
        localStorage.setItem('language', defaultLang);
        if (isDebug == 'true') {
            console.log('Languages->' + languages);
        }
    });
}

chrome.runtime.onInstalled.addListener(getPlatform);
chrome.runtime.onStartup.addListener(getPlatform);
chrome.runtime.onInstalled.addListener(getLanguage);
chrome.runtime.onStartup.addListener(getLanguage);

$.getJSON(apiUrl + '/services/opens/ips/format/json', function(data) {
    clientIP = data.data.ip;
    if (isDebug == 'true') {
        console.log('Background clientIP->' + clientIP);
    }
});

function getConfigUpdateTimeDuration() {
    var configStr = localStorage.getItem('config');
    var configData = JSON.parse(configStr);
    var configUpdateTime = configData.update_time;
    var configDuration = configData.request_duration;
    return { configUpdateTime, configDuration };
}

function getNodeUpdateTimeDuration() {
    var nodeStr = localStorage.getItem('node');
    var nodeData = JSON.parse(nodeStr);
    var nodeUpdateTime = nodeData.update_time;
    var nodeDuration = nodeData.request_duration;
    return { nodeUpdateTime, nodeDuration };
}

function getLoginStatus() {
    var domain = configData.data.base.root_url;
    chrome.cookies.get({url: 'http://' + domain, name: 'email'}, function(cookie) {
        if (cookie) {
            var email = cookie.value;
        }
        if(email){
            localStorage.setItem('isLogin', 'yes');
            var accessToken = localStorage.getItem('accessToken');
            if (!accessToken) {
                getAccessToken(email);
            }
        } else {
            localStorage.setItem('isLogin', 'no');
        }
    });
}

async function getAccessToken(email) {
    try {
        var paramStr;
        var accessTokenTemp = localStorage.getItem('accessToken');
        var userLevelTemp = localStorage.getItem('userLevel');
        if (email) {
            paramStr = '/email/' + email;
        }
        if (accessTokenTemp && userLevelTemp) {
            paramStr = paramStr + '/access_token/' + accessTokenTemp + '/user_level/' + userLevelTemp;
        }
        var requestUrl = apiUrl + '/services/opens/access_tokens/platform/' + platform + '/product_sn/' + productSn + '/version_id/' + version + paramStr + '/format/json';
        const response = await fetch(requestUrl);
        const data = await response.json();

        var res = data.data;
        accessToken = res.access_token;
        userLevel = res.user_level;

        localStorage.setItem('accessToken', accessToken);
        localStorage.setItem('userLevel', userLevel);

        if (isDebug == 'true') {
            console.log('Get access token succeeded');
            console.log(accessToken);
        }

        return res;
    } catch (error) {
        if (isDebug == 'true') {
            console.log('Failed to get access token');
        }

        return error;
    }
}

async function getConfig() {
    try {
        const response = await fetch(apiUrl + '/services/opens/configs/platform/' + platform + '/product_sn/' + productSn + '/version_id/' + version + '/format/json');
        const data = await response.json();
        configStr = JSON.stringify(data);

        localStorage.setItem('config', configStr);

        configData = res;
        proxyUrls = configData['data']['base']['match'];

        setApiUrl();
        getLoginStatus();
        setUsageDuration();

        if (isDebug == 'true') {
            console.log('Get configuration succeeded');
            console.log(configData);
            console.log(proxyUrls);
        }
    } catch (error) {
        configStr = localStorage.getItem('config');
        configData = JSON.parse(configStr);

        proxyUrls = configData['data']['base']['match'];

        setApiUrl();
        getLoginStatus();
        setUsageDuration();

        if (isDebug == 'true') {
            console.log('Failed to get configuration');
            console.log(configData);
            console.log(proxyUrls);
        }
    }
}
getConfig();

async function getNode() {
    try {
        const response = await fetch(apiUrl + '/services/opens/node_lists/product_sn/' + productSn + '/format/json');
        const data = await response.json();

        var nodeStr = JSON.stringify(data);

        localStorage.setItem('node', nodeStr);
    } catch (error) {
        //console.log('Failed to get node');
    }
}
getNode();

async function connectionNode() {
    try {
        accessToken = localStorage.getItem('accessToken');
        var authorizeUrl = proxyUrls.join(',');
        const response = await fetch(apiUrl + '/services/opens/connection_nodes/access_token/' + accessToken + '/product_sn/' + productSn + '/node_sn/b2c3c3ca-0605-cb80-91a5-fc337f48ecac/client_sn/' + clientId + '/client_ip/' + clientIP + '/authorize_url/' + authorizeUrl + '/format/json');
        const data = await response.json();

        const res = data;

        if (isDebug == 'true') {
            console.log('Connection node succeeded');
        }

        return res;
    } catch (error) {
        if (isDebug == 'true') {
            console.log('Failed to connect node');
        }

        return error;
    }
}

async function usageNode() {
    try {
        accessToken = localStorage.getItem('accessToken');
        switchStatus = localStorage.getItem('switchStatus');
        if (accessToken && switchStatus == 'on') {
            const response = await fetch(apiUrl + '/services/opens/usage_nodes/access_token/' + accessToken + '/product_sn/' + productSn + '/node_sn/b2c3c3ca-0605-cb80-91a5-fc337f48ecac/format/json');
            const data = await response.json();
            const res = data;
            if (data.status == true) {
                localStorage.setItem('userLevel', data.data.user_level);
            } else {
                localStorage.setItem('switchStatus', 'off');
                localStorage.setItem('pacScript', '');
                chrome.runtime.sendMessage({
                    type: 'toggle-proxy',
                    status: 'off'
                });
                localStorage.setItem('userLevel', data.data.user_level);
            }
            if (isDebug == 'true') {
                console.log('Successfully used node');
                console.log(res.msg);
                console.log(res);
            }
        } else {
            if (isDebug == 'true') {
                console.log('The parameter is empty');
            }
        }
    } catch (error) {
        if (isDebug == 'true') {
            console.log('Failed to use node');
            console.log(error);
        }
    }
}
setInterval(usageNode, 180 * 1000);

if (isDebug == 'true') {
    console.log(clientId);
    console.log(version);
}

var listener = function(details) {
    if (isDebug == 'true') {
        console.log('onBeforeRequest - Start: ' + '|' + status, details);
    }

    switchStatus = localStorage.getItem('switchStatus');
    var pacInfo = localStorage.getItem('pacScript');
    var proxyInfo = atob(pacInfo);
    if (proxyInfo) {
        var splitResult = proxyInfo.split(/:|\s+/);
        var protocol = splitResult[0].trim();
        var ip = splitResult[1].trim();
        var port = splitResult[2].trim();
    }
    if (switchStatus == 'off') {
        proxyUrls = [];
    }
    pacScriptStr = "function FindProxyForURL(url, host) {\n" +
    "    var proxyUrls = " + JSON.stringify(proxyUrls) + ";\n" +
    "    for (var i = 0; i < proxyUrls.length; i++) {\n" +
    "        if (shExpMatch(url, proxyUrls[i] + '*')) {\n" +
    "            return '" + protocol + " " + ip + ":" + port + "; DIRECT';\n" +
    "        }\n" +
    "    }\n" +
    "    console.log('Not using proxy for ' + url);\n" +
    "    return 'DIRECT';\n" +
    "}";
    proxyConfig = {
        mode: 'pac_script',
        pacScript: {
            data: pacScriptStr
        }
    };

    chrome.proxy.settings.set({value: proxyConfig, scope: "regular"}, function() {});

    if (isDebug == 'true') {
        console.log(proxyUrls);
        console.log([protocol, ip, port]);
    }
};

chrome.runtime.onMessage.addListener(function(request, sender, sendResponse) {
    if (request.type === 'toggle-proxy') {
        var status = request.status;
        if (isDebug == 'true') {
            console.log('onMessage - Start: ', status);
        }
        if (status === 'on') {
            chrome.webRequest.onBeforeRequest.addListener(
                listener,
                {urls: ["<all_urls>"]},
                ["blocking"]
            );
            if (isDebug == 'true') {
                console.log('onBeforeRequest - Listener Added');
            }
        } else {
            chrome.webRequest.onBeforeRequest.removeListener(listener);
            if (isDebug == 'true') {
                console.log('onBeforeRequest - Listener Removed');
            }
            chrome.proxy.settings.set({value: {mode: "direct"}, scope: "regular"}, function() {});
        }
    }
});

chrome.webRequest.onCompleted.addListener(
    function(details) {
        if (isDebug == 'true') {
            console.log('onCompleted - End: ' + '|' + status, details);
        }
    },
    {urls: ["<all_urls>"]}
);

