var bgWindow     = chrome.extension.getBackgroundPage();
var configStr    = localStorage.getItem('config');
var configData   = JSON.parse(configStr);
var nodeStr      = localStorage.getItem('node');
var nodeData     = JSON.parse(nodeStr);
var isLogin      = localStorage.getItem('isLogin');
var userLevel    = localStorage.getItem('userLevel');
var switchStatus = localStorage.getItem('switchStatus');
var switchButton = document.querySelector('#switch-connect input');
var defaultLang  = localStorage.getItem('language');
var isDebug      = bgWindow.getIsDebug();
var platform     = bgWindow.platform;
var clientIP;

chrome.storage.sync.get('language', function(items) {
    var lang = items.language || defaultLang;
    applyLanguage(lang);
});

function switchLanguage(lang) {
    chrome.storage.sync.set({'language': lang});
    applyLanguage(lang);
}

function applyLanguage(lang) {
    $("[data-translate]").each(function() {
        var key = $(this).data('translate');
        $(this).text(chrome.i18n.getMessage(key));
    });
}

$.getJSON(configData.data.api.url + '/services/opens/ips/format/json', function(data) {
    clientIP = data.data.ip;
    if (isDebug == 'true') {
        console.log('Popup clientIP->' + clientIP);
    }
});

chrome.runtime.getBackgroundPage(async function(bgWindow) {
    var {configUpdateTime, configDuration} = await bgWindow.getConfigUpdateTimeDuration();
    var now = new Date();
    var configLastUpdateTime = new Date(configUpdateTime);
    var configElapsedTime = (now - configLastUpdateTime) / 1000;
    if (configElapsedTime > configDuration) {
        await bgWindow.getConfig();
        if (isDebug == 'true') {
            console.log('Need to obtain configuration information again');
        }
        var version = bgWindow.version;
        var lastVersion = configData['data']['base']['version'][platform];
        if(version !== lastVersion) {
            var versionTips = chrome.i18n.getMessage('version_tips');
            versionTips = versionTips.replace('{VERSION}', lastVersion);
            showAlert(versionTips, 15000);
            DelayVisit(configData['data']['base']['product_url'], 2000);
        }
    } else {
        if (isDebug == 'true') {
            console.log('No need to update configuration information->' + configUpdateTime + '|' + configDuration );
        }
    }
    var {nodeUpdateTime, nodeDuration} = await bgWindow.getNodeUpdateTimeDuration();
    var now = new Date();
    var nodeLastUpdateTime = new Date(nodeUpdateTime);
    var nodeElapsedTime = (now - nodeLastUpdateTime) / 1000; // 转换为秒
    if (nodeElapsedTime > nodeDuration) {
        await bgWindow.getNode();
        if (isDebug == 'true') {
            console.log('Need to obtain node information again');
        }
    } else {
        if (isDebug == 'true') {
            console.log('Node information does not need to be updated->' + nodeUpdateTime + '|' + nodeDuration);
        }
    }
});

var domain = configData.data.base.root_url;
$(function() {
    document.title = configData.data.base.name_en;
    $('#nick-name').attr('data-default', chrome.i18n.getMessage('login'));
    chrome.cookies.get({url: 'http://' + domain, name: 'email'}, function(cookie) {
        if (cookie) {
            localStorage.setItem('isLogin', 'yes');
            var nickName = cookie.value;
            chrome.runtime.getBackgroundPage(async function(bgWindow) {
                //令牌更新
                var res = await bgWindow.getAccessToken(nickName);
                if (isDebug == 'true') {
                    console.log('Update access token->' + res.access_token + '|' + res.msg);
                }
            });
            $('#nick-name').text(nickName).attr('title', unescape(nickName));
            if (userLevel) {
                $('.user-level').text(userLevel);
            } else {
                $('.user-level').text($('.user-level').attr('data-default'));
            }
            $('#user-level-login').show();
            $('#user-level-nologin').hide();
        } else {
            localStorage.setItem('isLogin', 'no');
            localStorage.setItem('switchStatus', 'off');
            chrome.runtime.sendMessage({
                type: 'toggle-proxy',
                status: 'off'
            });
            $('#nick-name').text($('#nick-name').attr('data-default'));
            $('.user-level').text($('.user-level').attr('data-default'));
            $('#user-level-login').hide();
            $('#user-level-nologin').show();
        }
    });

    function displayNode (nodeData) {
        if (nodeData.data) {
            const selectNode = document.getElementById('select-node');
            const dropdownMenu = selectNode.querySelector('.dropdown-menu');
            const dropdownSelect = document.getElementById('dropdown-select');
            dropdownSelect.setAttribute('data-type', nodeData.data[0].type_simple);
            dropdownSelect.setAttribute('data-value', nodeData.data[0].identify_sn);
            dropdownSelect.innerHTML = `
        <img src="${nodeData.data[0].flag_img}" width="18" height="14" style="position:relative;top:-2px;" />
            <span class="selected-node-name">&nbsp;${nodeData.data[0].country} ${nodeData.data[0].city} #${nodeData.data[0].node_id} ${nodeData.data[0].type}</span>
    `;
            const nodes = nodeData.data.sort((a, b) => a.sort - b.sort);
            const firstLi = dropdownMenu.querySelector('li:first-child');
            nodes.forEach((node) => {
                const imgSrc = node.flag_img;
                const nodeType = node.type;
                const nodeTypeSimple = node.type_simple;
                const nodeValue = node.identify_sn;
                const nodeName = ` ${node.country} ${node.city} #${node.node_id} `;
                const li = document.createElement('li');
                const a = document.createElement('a');
                a.classList.add('dropdown-item', 'h6', 'text-muted');
                a.setAttribute('data-type', nodeTypeSimple);
                a.setAttribute('data-value', nodeValue);
                const img = document.createElement('img');
                img.setAttribute('src', imgSrc);
                a.appendChild(img);
                const spanName = document.createElement('span');
                spanName.classList.add('node-name');
                spanName.style.position = 'relative';
                spanName.style.top = '1px';
                spanName.textContent = nodeName;
                a.appendChild(spanName);
                const spanType = document.createElement('span');
                spanType.classList.add('node-type', 'badge', 'rounded-pill');
                spanType.textContent = nodeType;
                if (nodeType === 'Trial') {
                    spanType.classList.add('bg-primary', 'opacity-75');
                } else if (nodeType === 'Professional') {
                    spanType.classList.add('bg-dark', 'opacity-75');
                }
                spanType.style.position = 'relative';
                spanType.style.top = '-1px';
                a.appendChild(spanType);

                li.appendChild(a);
                dropdownMenu.appendChild(li);
            });
        }
    }
    if (!nodeData) {
        if (isDebug == 'true') {
            console.log('Get node data again');
        }
        bgWindow.getNode();
    }
    if (isDebug == 'true') {
        console.log(nodeData);
    }
    displayNode(nodeData);
    var switchNode = localStorage.getItem('switchNode');
    if (switchNode) {
        $('#select-node .dropdown-item[data-value="' + switchNode + '"]').trigger('click');
    }
})

if (configData['data']['advert']['popup']['a1']['display']['trial'] == 'true') {
    if (!userLevel) {
        userLevel = 'TRY';
    }
    const userLevelTemp = userLevel.toLowerCase();
    const upgradeTipsDiv = document.getElementById('upgrade-tips');
    const upgradeTipsLink = upgradeTipsDiv.querySelector('a');
    if (defaultLang.indexOf('zh-') !== -1) {
        var advertTitle = configData['data']['advert']['popup']['a1'][userLevelTemp + '_title_zh'];
        var advertPointText = configData['data']['advert']['popup']['a1'][userLevelTemp + '_point_text_zh'];
    } else {
        var advertTitle = configData['data']['advert']['popup']['a1'][userLevelTemp + '_title_en'];
        var advertPointText = configData['data']['advert']['popup']['a1'][userLevelTemp + '_point_text_en'];
    }
    upgradeTipsLink.innerHTML = advertTitle;
    upgradeTipsLink.innerHTML = upgradeTipsLink.innerHTML.replace(advertPointText, '<span class="text-decoration-underline">' + advertPointText + '</span>');
    let originalStyles = getComputedStyle(upgradeTipsDiv);
    let increaseStyles = configData['data']['advert']['popup']['a1']['div_style'];
    upgradeTipsDiv.classList.add(configData['data']['advert']['popup']['a1']['div_class']);
    upgradeTipsLink.classList.add(configData['data']['advert']['popup']['a1']['a_class']);
    upgradeTipsDiv.style.cssText = `
    ${originalStyles.cssText}
    ${increaseStyles}
`;
}
let commonServices = document.querySelectorAll('.common-services');
commonServices.forEach(function(element) {
    element.textContent = chrome.i18n.getMessage('common_services');
});
var recommendSiteData = configData['data']['link']['recommend'];
const recommendSite = document.getElementById('recommend-site');
for (let i = 0; i < recommendSiteData.length; i++) {
    const site = recommendSiteData[i];
    const div = document.createElement('div');
    const a = document.createElement('a');
    const img = document.createElement('img');
    div.style.height = '32px';
    if (site.url) {
        a.href = site.url;
        a.classList.add('link-info', 'text-decoration-none');
        a.target = '_blank';
        img.src = site.logo;
        img.width = '24';
        img.height = '24';
        img.style.position = 'relative';
        img.style.top = '-2px';
        a.appendChild(img);
        a.appendChild(document.createTextNode('' + site.title_en_short));
        div.appendChild(a);
    } else {
        div.style.width = '0px';
    }
    recommendSite.appendChild(div);
}
let siteUrls = document.querySelectorAll('.site-url');
siteUrls.forEach(siteUrl => siteUrl.href = configData['data']['base']['url']);
let siteNames = document.querySelectorAll('.site-name');
siteNames.forEach(function(element) {
  element.textContent = configData['data']['base']['brand_name_en'];
});
let organizationTypes = document.querySelectorAll('.organization-type');
organizationTypes.forEach(function(element) {
  element.textContent = configData['data']['base']['organization_en'];
});
let projectNames = document.querySelectorAll('.project-name');
projectNames.forEach(projectNames => projectNames.href = configData['data']['base']['product_url']);
projectNames.forEach(function(element) {
  element.textContent = configData['data']['base']['name_en'];
});
let githubUrls = document.querySelectorAll('.github-url');
githubUrls.forEach(githubUrls => githubUrls.href = configData['data']['base']['github']);
var targetUrl = (isLogin == 'yes') ? configData['data']['link']['home']['user_dashboard'] : configData['data']['link']['home']['login'];
var subscriptionPrice = (isLogin == 'yes') ? configData['data']['link']['home']['subscription_price2'] : configData['data']['link']['home']['subscription_price'];
let userlogins = document.querySelectorAll('.user-login');
userlogins.forEach(userlogins => userlogins.href = targetUrl);
let userCenters = document.querySelectorAll('.user-center');
userCenters.forEach(userCenters => userCenters.href = configData['data']['link']['home']['user_dashboard']);
userCenters.forEach(function(element) {
    element.textContent = chrome.i18n.getMessage('user_center');
});
let upgradeProfessionals = document.querySelectorAll('.upgrade-professional');
let upgradeProfessionalPackage = document.querySelectorAll('.upgrade-professional-package');
upgradeProfessionals.forEach(upgradeProfessionals => upgradeProfessionals.href = subscriptionPrice);
upgradeProfessionalPackage.forEach(function(element) {
    if (userLevel == 'PRO') {
        element.textContent = chrome.i18n.getMessage('upgrade_professional_package2');
    } else {
        element.textContent = chrome.i18n.getMessage('upgrade_professional_package');
    }
});
let expansionPackages = document.querySelectorAll('.expansion-package');
expansionPackages.forEach(expansionPackages => expansionPackages.href = configData['data']['link']['home']['expansion_price']);
expansionPackages.forEach(function(element) {
    element.textContent = chrome.i18n.getMessage('expansion_package');
});
let logout = document.querySelectorAll('.logout');
logout.forEach(function(element) {
    element.textContent = chrome.i18n.getMessage('logout');
});

var connectText        = '#connect-text';
var loadingIcon        = '.loading-icon';
var connectWaitIcon    = '#connect-wait-icon';
var connectLoadingIcon = '#connect-loading-icon';
var connectSuccessIcon = '#connect-success-icon';
var connectFailIcon    = '#connect-fail-icon';
if (isLogin == 'yes') {
    if (switchStatus === 'on') {
        chrome.runtime.sendMessage({
            type: 'toggle-proxy',
            status: 'on'
        });
        $(connectText).text(chrome.i18n.getMessage('connect_success'));
        $(loadingIcon).addClass('d-none');
        $(connectSuccessIcon).removeClass('d-none').addClass('d-block');
        switchButton.checked = true;
    } else {
        chrome.runtime.sendMessage({
            type: 'toggle-proxy',
            status: 'off'
        });
        $(connectText).text(chrome.i18n.getMessage('connect_wait'));
        $(loadingIcon).addClass('d-none');
        $(connectWaitIcon).removeClass('d-none').addClass('d-block');
        switchButton.checked = false;
    }
} else {
    $(connectText).text(chrome.i18n.getMessage('connect_wait'));
    $(loadingIcon).addClass('d-none');
    $(connectWaitIcon).removeClass('d-none').addClass('d-block');
    switchButton.checked = false;
}

$('#switch-connect').on('click', function(){
    var isLogin = localStorage.getItem('isLogin');
    var isChecked = $('#switch-connect input:checkbox').is(':checked');
    var status = isChecked ? 'on' : 'off';
    if (isLogin !== 'yes') {
        $('#switch-connect input:checkbox').prop('checked', false);
        showAlert(chrome.i18n.getMessage('nologin_connect_error'), 5000);
        return;
    }
    if (isLogin == 'yes' && status == 'on') {
        var connectionStatus;
        chrome.runtime.getBackgroundPage(async function(bgWindow) {
            var res = await bgWindow.connectionNode();
            if (isDebug == 'true') {
                console.log('Start connecting nodes->');
                console.log(res);
            }
            if (res.status == false) {
                connectionStatus = false;
                localStorage.setItem('switchStatus', 'off');
                chrome.runtime.sendMessage({
                    type: 'toggle-proxy',
                    status: 'off'
                });
                $('#switch-connect input:checkbox').prop('checked', false);
                showAlert(chrome.i18n.getMessage('connection_node_code' + res.code), 5000);
                if (isDebug == 'true') {
                    console.log('Failed to connect node->' + res.msg);
                }
                return false;
            } else {
                if (res.data.node_script) {
                    localStorage.setItem('pacScript', res.data.node_script);
                }
                connectionStatus = true;
                localStorage.setItem('switchStatus', 'on');
                chrome.runtime.sendMessage({
                    type: 'toggle-proxy',
                    status: 'on'
                });
                if (isDebug == 'true') {
                    console.log('Connection node succeeded');
                }
            }
        });
        $(connectText).text(chrome.i18n.getMessage('connect_wait'));
        $(loadingIcon).addClass('d-none');
        $(connectWaitIcon).removeClass('d-none').addClass('d-block');
        setTimeout(function() {
            $(connectText).text(chrome.i18n.getMessage('loading'));
            $(loadingIcon).addClass('d-none');
            $(connectLoadingIcon).removeClass('d-none').addClass('d-block');
        }, 1000);
        setTimeout(function() {
            if (connectionStatus) {
                $(connectText).text(chrome.i18n.getMessage('connect_success'));
            } else {
                $(connectText).text(chrome.i18n.getMessage('connect_fail'));
            }
            $(loadingIcon).addClass('d-none');
            if (connectionStatus) {
                $(connectSuccessIcon).removeClass('d-none').addClass('d-block');
            } else {
                $(connectFailIcon).removeClass('d-none').addClass('d-block');
            }
        }, 2000);
    } else {
        connectionStatus = false;
        localStorage.setItem('switchStatus', 'off');
        chrome.runtime.sendMessage({
            type: 'toggle-proxy',
            status: 'off'
        });
        $(connectText).text(chrome.i18n.getMessage('connect_close'));
        $(loadingIcon).addClass('d-none');
        $(connectLoadingIcon).removeClass('d-none').addClass('d-block');
        setTimeout(function() {
            $(connectText).text(chrome.i18n.getMessage('connect_wait'));
            $(loadingIcon).addClass('d-none');
            $(connectWaitIcon).removeClass('d-none').addClass('d-block');
        }, 1000);
    }
    if (isDebug == 'true') {
        console.log(isChecked + '|' + status);
        if (isChecked) {
            console.log('client IP: ' + clientIP + ' on proxy!');
        } else {
            console.log('client IP: ' + clientIP + ' off proxy!');
        }
    }
});

$(document).on('click', '#user-level-login', function() {
    $('#user-dropdown').css('margin-top', '21px');
});

$(document).on('click', '#select-node .dropdown-item', function() {
    var type = $(this).attr('data-type').toUpperCase();
    var value = $(this).attr('data-value');
    var imgSrc = $(this).find('img').attr('src');
    if(userLevel == 'TRY' && type !== userLevel) {
        $('#switch-connect input:checkbox').prop('checked', false);
        showAlert(chrome.i18n.getMessage('trial_select_node_error'), 5000);
        if (isDebug == 'true') {
            console.log('Node selected value: ' + userLevel + '|' + type + '|' + value + ', img src: ' + imgSrc);
        }
        return;
    }
    localStorage.setItem('switchNode', value);
    $('#dropdown-select').text($(this).text()).attr('data-type', type).attr('data-value', value).prepend('<img src="' + imgSrc + '" width="18" height="14" style="position:relative;top:-2px;">&nbsp;');
    if (isDebug == 'true') {
        console.log('Node selected value: ' + userLevel + '|' + type + '|' + value + ', img src: ' + imgSrc);
    }
});

$('#sign-out').on('click', function() {
    chrome.cookies.getAll({domain: domain}, function(cookies) {
        for (var i = 0; i < cookies.length; i++) {
            chrome.cookies.remove({url: 'http://' + cookies[i].domain + cookies[i].path, name: cookies[i].name});
        }
    });
    $.ajax({
        type: 'GET',
        url: 'http://' + domain + '/logout',
        success: function(response) {
            if (isDebug == 'true') {
                console.log('Successfully logged out');
            }
        }
    });
    localStorage.setItem('isLogin', 'no');
    localStorage.setItem('switchStatus', 'off');
    chrome.runtime.sendMessage({
        type: 'toggle-proxy',
        status: 'off'
    });
    $('#nick-name').text($('#nick-name').attr('data-default')).attr('title', '');
    $('.user-level').text($('.user-level').attr('data-default'));
    $('#user-level-login').hide();
    $('#user-level-nologin').show();
    $('.user-login').attr('href', configData['data']['link']['home']['login']);
});

$('.alert-close').on('click', function(){
    $('.alert-text').text('');
    $('.alert-tips').removeClass('d-block').addClass('d-none');
});

function DelayVisit(url, duration) {
    setTimeout(function() {
        chrome.tabs.create({url:url});
    }, duration);
}

function showAlert(text, duration) {
    $('.alert-text').text(text);
    $('.alert-tips').removeClass('d-none').addClass('d-block');
    setTimeout(function() {
        closeAlert();
    }, duration);
}

function closeAlert() {
    $('.alert-close').click();
}

